package com.dayou.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dayou.dto.AssetsStatisticalSelectDTO;
import com.dayou.entity.AssetsEvaluationMethod;
import com.dayou.entity.User;
import com.dayou.mapper.AssetsEvaluationMethodMapper;
import com.dayou.mapper.AssetsStatisticalStatementMapper;
import com.dayou.service.IAssetsStatisticalStatementService;
import com.dayou.service.IUserService;
import com.dayou.vo.AssetsLedgerVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
public class AssetsStatisticalStatementServiceImpl implements IAssetsStatisticalStatementService {

    @Autowired
    private AssetsStatisticalStatementMapper assetsStatisticalStatementMapper;

    @Autowired
    private AssetsEvaluationMethodMapper assetsEvaluationMethodMapper;

    @Autowired
    private IUserService userService;

    /**
     * 获取资产台账
     *
     * @param dto 查询dto
     * @return Page<AssetsLedgerVO>
     */
    @Override
    public Page<AssetsLedgerVO> getAssetsLedgerVO(AssetsStatisticalSelectDTO dto, Page page) {
        Page<AssetsLedgerVO> assetsLedgerVOPage = assetsStatisticalStatementMapper.getAssetsLedgerVO(page, dto);
        // 获取评估方法信息
        List<AssetsEvaluationMethod> evaluationMethodList = assetsEvaluationMethodMapper.getAssetsEvaluationMethod();
        // 获取用户信息
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<User>()
                .select(User::getId,User::getName)
                .eq(User::getDeleted,false);
        List<User> userList= userService.list(lambdaQueryWrapper);
        // 设置评估方法与签字人等信息
        for (AssetsLedgerVO assetsLedgerVO : assetsLedgerVOPage.getRecords()) {
            assetsLedgerVO = setVOJSONInfo(assetsLedgerVO, evaluationMethodList, userList);
        }
        return assetsLedgerVOPage;
    }

    /**
     * 资产台账导出
     * @param dto 查询dto
     * @return Page<AssetsLedgerVO>
     */
    @Override
    public List<AssetsLedgerVO> assetsLedgerVOExport(AssetsStatisticalSelectDTO dto) {
        List<AssetsLedgerVO> assetsLedgerVOList = assetsStatisticalStatementMapper.assetsLedgerVOExport(dto);
        // 获取评估方法信息
        List<AssetsEvaluationMethod> evaluationMethodList = assetsEvaluationMethodMapper.getAssetsEvaluationMethod();
        // 获取用户信息
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<User>()
                .select(User::getId,User::getName)
                .eq(User::getDeleted,false);
        List<User> userList= userService.list(lambdaQueryWrapper);
        // 设置评估方法与签字人等信息
        for (AssetsLedgerVO assetsLedgerVO : assetsLedgerVOList) {
            assetsLedgerVO = setVOJSONInfo(assetsLedgerVO, evaluationMethodList, userList);
        }
        return assetsLedgerVOList;
    }

    /**
     * 设置评估方法与签字人等信息
     * @param assetsLedgerVO 查询出来的VO信息
     * @param evaluationMethodList 评估方法信息
     * @param userList 用户信息
     * @return AssetsLedgerVO
     */
    public AssetsLedgerVO setVOJSONInfo(AssetsLedgerVO assetsLedgerVO, List<AssetsEvaluationMethod> evaluationMethodList, List<User> userList){
        // 正则表达式匹配任何在[]内的值
        Pattern pattern = Pattern.compile("\\[(.*?)\\]");

        // 获取评估方法
        if (ObjectUtil.isNotNull(assetsLedgerVO.getEvaluationMethods())){
            Matcher matcher = pattern.matcher(assetsLedgerVO.getEvaluationMethods());
            List<String> methodIdsList = new ArrayList<>();
            while (matcher.find()) {
                // 将匹配的整个字符串添加到列表中
                methodIdsList.add(matcher.group(0));
            }
            // 新建字符串用来装处理后的方法名
            String evaluationMethodNames = null;
            // 获取方法id
            for (String methodIds : methodIdsList) {
                // 将JSON格式的id数组转成集合
                JSONArray jsonArray = JSONArray.parseArray(methodIds);
                List<Long> idList = jsonArray.toJavaList(Long.class);
                // 通过id进行匹配并返回方法名
                int index = 0;
                for (AssetsEvaluationMethod evaluationMethod : evaluationMethodList) {
                    if (idList.contains(evaluationMethod.getId())){
                        if (ObjectUtil.isNotNull(evaluationMethodNames)){
                            evaluationMethodNames += evaluationMethod.getMethodName();
                            index++;
                        }else {
                            evaluationMethodNames = evaluationMethod.getMethodName();
                            index++;
                        }
                        if (index < idList.size()){
                            evaluationMethodNames += '，';
                        }else {
                            evaluationMethodNames += '；';
                        }
                    }
                }
            }
            assetsLedgerVO.setEvaluationMethods(evaluationMethodNames.toString());
        }

        // 获取最终选择的评估方法
        if (ObjectUtil.isNotNull(assetsLedgerVO.getChoiceEvaluationMethods())){
            Matcher choiceMatcher = pattern.matcher(assetsLedgerVO.getChoiceEvaluationMethods());
            List<String> choiceMethodIdsList = new ArrayList<>();
            while (choiceMatcher.find()) {
                // 将匹配的整个字符串添加到列表中
                choiceMethodIdsList.add(choiceMatcher.group(0));
            }
            // 新建字符串用来装处理后的方法名
            String choiceEvaluationMethodNames = null;
            // 获取方法id
            for (String methodIds : choiceMethodIdsList) {
                // 将JSON格式的id数组转成集合
                JSONArray jsonArray = JSONArray.parseArray(methodIds);
                List<Long> idList = jsonArray.toJavaList(Long.class);
                // 通过id进行匹配并返回方法名
                int index = 0;
                for (AssetsEvaluationMethod evaluationMethod : evaluationMethodList) {
                    if (idList.contains(evaluationMethod.getId())){
                        if (ObjectUtil.isNotNull(choiceEvaluationMethodNames)){
                            choiceEvaluationMethodNames += evaluationMethod.getMethodName();
                            index++;
                        }else {
                            choiceEvaluationMethodNames = evaluationMethod.getMethodName();
                            index++;
                        }
                        if (index < idList.size()){
                            choiceEvaluationMethodNames += '，';
                        }else {
                            choiceEvaluationMethodNames += '；';
                        }
                    }
                }
            }
            assetsLedgerVO.setChoiceEvaluationMethods(choiceEvaluationMethodNames.toString());
        }

        // 获取签字人
        if (ObjectUtil.isNotNull(assetsLedgerVO.getSignatory())){
            String signatoryNames = null;
            JSONArray jsonArray = JSONArray.parseArray(assetsLedgerVO.getSignatory());
            List<Long> signatoryIdList = jsonArray.toJavaList(Long.class);
            int index = 0;
            for (User user : userList){
                if (signatoryIdList.contains(user.getId())){
                    if (ObjectUtil.isNotNull(signatoryNames)){
                        signatoryNames += user.getName();
                        index++;
                    }else {
                        signatoryNames = user.getName();
                        index++;
                    }
                    if (index < signatoryIdList.size()){
                        signatoryNames += '，';
                    }else {
                        signatoryNames += '；';
                    }
                }
            }
            assetsLedgerVO.setSignatory(signatoryNames);
        }

        return assetsLedgerVO;
    }
}
